/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "fram.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define FRAM_DUMMY 0

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void fram_cfg_setup ( fram_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->hld = HAL_PIN_NC;
    cfg->wp = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

FRAM_RETVAL fram_init ( fram_t *ctx, fram_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return FRAM_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, FRAM_DUMMY );

    // Output pins 
    
    digital_out_init( &ctx->hld, cfg->hld );
    digital_out_init( &ctx->wp, cfg->wp );

    return FRAM_OK;

}

void fram_generic_transfer ( fram_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start( &ctx->spi );
    spi_master_transfer( &ctx->spi, block );
    spi_master_stop( &ctx->spi );    
}

void fram_init_f ( fram_t *ctx ) 
{
    spi_master_stop(&ctx->spi);
}

void fram_write_enable ( fram_t *ctx ) 
{
    uint8_t temp;
    temp = FRAM_WREN;
    
    spi_master_start( &ctx->spi );
    spi_master_write( &ctx->spi, temp );
    spi_master_stop( &ctx->spi ); 
}

void fram_write_disable ( fram_t *ctx ) 
{
    uint8_t temp;
    temp = FRAM_WRDI;

    spi_master_start( &ctx->spi );
    spi_master_write( &ctx->spi, temp );
    spi_master_stop( &ctx->spi );     
  
    return;
}

uint8_t fram_read_status ( fram_t *ctx ) 
{
    spi_master_transfer_data_t block;
    uint8_t temp[ 2 ];
    uint8_t reg;

    reg = FRAM_RDSR;
    
    block.tx_buffer = &reg; 
    block.rx_buffer = &temp;
    block.tx_length = 1;
    block.rx_length = 2;

    spi_master_start( &ctx->spi );
    spi_master_transfer( &ctx->spi, &block );
    spi_master_stop( &ctx->spi ); 
  
    return temp[ 1 ];
}

void fram_write_status ( fram_t *ctx, uint8_t value ) 
{
    spi_master_transfer_data_t block;
    uint8_t temp[ 2 ];

    temp[ 0 ] = FRAM_WRSR;
    temp[ 1 ] = value;

    fram_write_enable( ctx );
    
    spi_master_start( &ctx->spi );
    spi_master_write( &ctx->spi, temp[ 0 ] );
    spi_master_write( &ctx->spi, temp[ 1 ] );
    spi_master_stop( &ctx->spi ); 
}

void fram_read ( fram_t *ctx, uint16_t address, uint8_t *buffer, uint16_t count ) 
{   
    spi_master_transfer_data_t block;
    uint8_t *data_ptr;
    uint8_t temp[ 3 ];
    uint8_t i;

    data_ptr  = buffer;

    temp[ 0 ]  = FRAM_READ;
    temp[ 1 ] = (uint8_t)( address >> 8 );
    temp[ 2 ] = (uint8_t)( address & 0x00FF );

    fram_write_enable( ctx ); 

    block.tx_buffer = &temp; 
    block.rx_buffer = &data_ptr;
    block.tx_length = 3;
    block.rx_length = count + 3;

    spi_master_start( &ctx->spi );
    spi_master_transfer( &ctx->spi, &block );
    spi_master_stop( &ctx->spi );   

    for ( i = 0 ; i < count; i++ )
    {
        buffer[ i ] =  data_ptr[ i + 3 ];
    }  
}

void fram_write ( fram_t *ctx, uint16_t address, uint8_t *buffer, uint16_t count ) 
{
    
    uint8_t *data_ptr;
    uint8_t temp[ 3 ];
    uint8_t i;

    data_ptr  = buffer;
    
    temp[ 0 ] = FRAM_WRITE;
    temp[ 1 ] = (uint8_t)( address >> 8 ) ;
    temp[ 2 ] = (uint8_t)( address & 0x00FF );
    
    fram_write_enable( ctx );

    spi_master_start( &ctx->spi );
    spi_master_write( &ctx->spi, temp[ 0 ] );
    spi_master_write( &ctx->spi, temp[ 1 ] );
    spi_master_write( &ctx->spi, temp[ 2 ] );

    for( i = 0; i < count; i++ )
    {
        spi_master_write( &ctx->spi, data_ptr[ i ] );
    }

    spi_master_stop( &ctx->spi );   
}

void fram_erase_all ( fram_t *ctx ) 
{
    uint16_t cntr;
    uint8_t temp[ 3 ];

    temp[ 0 ] = FRAM_WRITE;
    temp[ 1 ] = 0x00;
    temp[ 2 ] = 0x00;

    fram_write_status( ctx, 0x00 );
    fram_write_enable( ctx );
    
    spi_master_start( &ctx->spi );

    spi_master_write( &ctx->spi, temp[ 0 ] );    
    spi_master_write( &ctx->spi, temp[ 1 ] );
    spi_master_write( &ctx->spi, temp[ 2 ] );

    temp[ 0 ] = 0x00;

    for( cntr=0; cntr <= FRAM_MEM_SIZE; cntr++ )
    {
        spi_master_write( &ctx->spi, temp[ 0 ] );
    }
    
    spi_master_stop( &ctx->spi );

    fram_write_disable( ctx );  
}

// ------------------------------------------------------------------------- END

